#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""LombardPress print.
"""

import logging
import lxml
import os
import queue
import re
import subprocess
import threading
import urllib

import lbppy

MODULE_DIR = os.path.dirname(__file__)

class Transcription:
    """The main object of the script, defining the properties of the text under processing."""

    def __init__(self, input_argument):
        self.input = input_argument

    def get_schema_info(self):
        """Return schema version info."""
        pass

    def _define_file(self):
        """Return file object.
        """
        pass


class LocalTranscription(Transcription):
    """Object for handling local files."""

    def __init__(self, input):

        Transcription.__init__(self, input)
        self.file = self._define_file()
        self.lbp_schema_info = self.get_schema_info()
        logging.debug("Local resource initialized.")

    def get_schema_info(self):
        """Return the validation schema version."""
        # TODO: We need validation of the xml before parsing it. This is necesssary for proper user feedback on errors.

        try:
            schemaref_number = lxml.etree.parse(self.file.name).xpath(
                "/tei:TEI/tei:teiHeader[1]/tei:encodingDesc[1]/tei:schemaRef[1]/@n",
                namespaces={"tei": "http://www.tei-c.org/ns/1.0"}
                )[0]  # The returned result is a list. Grab first element.
            return {
                'version': schemaref_number.split('-')[2],
                'type': schemaref_number.split('-')[1]
            }
        except IndexError as e:
            logging.error('The document does not seem to contain a value in '
                          'TEI/teiHeader/encodingDesc/schemaRef[@n]. See the LombardPress documentation for help. '
                          'If the problem persists, please submit an issue report.')
            raise
        except Exception as e:
            logging.error('The process resulted in an error: {}.\n '
                          'If the problem persists, please submit an issue report.'.format(e))
            raise


    def _define_file(self):
        """Return the file object.
        """
        file_argument = self.input
        if os.path.isfile(file_argument):
            return open(file_argument)
        else:
            raise IOError(f"The supplied argument ({file_argument}) is not a file.")


class RemoteTranscription(Transcription):
    """Object for handling remote transcriptions.

    Keyword arguments:
    input -- SCTA resource id of the text to be processed.
    """

    def __init__(self, resource_input, download_dir=False):
        Transcription.__init__(self, resource_input)
        self.download_dir = download_dir
        self.resource = self.find_remote_resource(resource_input)
        self.direct_transcription = False  # Does the input refer directly to a transcription.
        self.transcription_object = self.define_transcription_object()
        self.id = self.input.split('/')[-1]
        self.file = self._define_file()
        self.lbp_schema_info = self.get_schema_info()
        logging.debug("Remote resource initialized.")

    def get_schema_info(self):
        """Return the validation schema version."""
        if self.direct_transcription:
            return {
                'version': self.transcription_object.file().validating_schema_version(),
                'type': self.transcription_object.transcription_type()
            }
        else:
            return {
                'version': self.transcription_object.resource().file().validating_schema_version(),
                'type': self.transcription_object.resource().transcription_type()
            }

    def find_remote_resource(self, resource_input):

        url_match = re.match(r'(http://)?(scta.info/resource)?', resource_input)
        url_string = ''
        if url_match:
            if url_match.group(1) is None:
                url_string += 'http://'
            if not url_match.group(2):
                url_string += 'scta.info/resource/'

        url_string += resource_input
        try:
            return lbppy.Resource.find(url_string)
        except AttributeError:
            logging.error(f'A resource with the provided ID ("{url_string}") could not be located. '
                          'Ensure that you have entered the correct id. ')
            raise

    def define_transcription_object(self):
        """
        Return a canonical transcription of either Manifestation (critical) or Expression (
        diplomatic) objects.
        """
        if isinstance(self.resource, lbppy.Expression):
            return self.resource.canonical_manifestation().resource().canonical_transcription()
        elif isinstance(self.resource, lbppy.Manifestation):
            return self.resource.canonical_transcription()
        elif isinstance(self.resource, lbppy.Transcription):
            self.direct_transcription = True
            return self.resource

    def _define_file(self):
        """Determine whether the file input supplied is local or remote and return its file object.
        """
        if self.download_dir:
            download_dir = self._find_or_create_download_dir(self.download_dir)
        else:
            download_dir = self._find_or_create_download_dir('download')

        file_path = os.path.join(download_dir, self.id + '.xml')
        # We are disabling the caching temporarily until we have a better solution.
        # if os.path.isfile(file_path):
        #     logging.info(f"Using cached version of ID {self.id}.")
        #     return open(file_path)
        # else:

        logging.info("Downloading remote resource...")
        if self.direct_transcription:
            url_object = self.transcription_object.file().file().geturl()
        else:
            url_object = self.transcription_object.resource().file().file().geturl()

        with urllib.request.urlopen(url_object) as response:
            transcription_content = response.read().decode('utf-8')
            with open(file_path, mode='w') as f:
                f.write(transcription_content)
        logging.info("Download of remote resource finished.")
        return open(f.name)

    def _find_or_create_download_dir(self, download_dir):
        if os.path.isdir(download_dir):
            return download_dir
        else:
            logging.warn(f'The supplied download directory {download_dir} does not exist. It will be created.')
            os.mkdir(download_dir)
            return download_dir


def convert_xml_to_tex(xml_file, xslt_script, output=False, xslt_parameters=False):
    """Convert the list of encoded files to tex, using the auxiliary XSLT script.

    The function creates a output dir in the current working dir and puts the tex file in that
    directory. The function requires saxon installed.

    Keyword Arguments:
    xml_buffer -- the content of the xml file under conversion
    xslt_script -- the content of the xslt script used for the conversion

    Return: File object.
    """
    logging.info(f"Start conversion of {xml_file}...")

    if xslt_parameters:
        process = subprocess.Popen(['java', '-jar', os.path.join(MODULE_DIR, 'vendor/saxon9he.jar'),
                                    f'-s:{xml_file}', f'-xsl:{xslt_script}', xslt_parameters],
                                   stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    else:
        process = subprocess.Popen(['java', '-jar', os.path.join(MODULE_DIR, 'vendor/saxon9he.jar'),
                                    f'-s:{xml_file}', f'-xsl:{xslt_script}'],
                                   stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    out, err = process.communicate()

    if err:
        logging.warning('The XSLT script reported the following warning(s):\n'
                        + err.decode('utf-8'))
    tex_buffer = out.decode('utf-8')

    # Output dir preparation: If output flags, check that dir and set, if not,
    # create or empty the dir "output" in current working dir.
    if output:
        if os.path.isdir(output):
            output_dir = output
        else:
            logging.warn(f'The supplied output directory {output} does not exist. It will be created.')
            os.mkdir(output)
            output_dir = output
    else:
        output_dir = 'output'
        if not output_dir in os.listdir('.'):
            os.mkdir(output_dir)
        else:
            for (root, dirs, files) in os.walk(output_dir):
                for name in files:
                    os.remove(os.path.join(root, name))

    # Output file name based on transcription object.
    basename, _ = os.path.splitext(os.path.basename(xml_file))
    with open(os.path.join(output_dir, basename + '.tex'), mode='w', encoding='utf-8') as f:
        f.write(tex_buffer)
    logging.info('The XML was successfully converted.')
    return f


def clean_tex(tex_file):
    """Clean the content of the tex file for different whitespace problems.

    Side effect: This changes the content of the file.

    :param tex_file: File object of the tex file.
    :return: File object of the tex file after cleanup.
    """
    logging.info("Trying to remove whitespace...")
    patterns = [
        (r' ?{ ?', r'{'),                 # Remove redundant space around opening bracket.
        (r' }', r'}'),                    # Remove redundant space before closing bracket.
        (r' ([.,?!:;])', r'\1'),          # Remove redundant space before punctuation.
        (r' (\\edtext{})', r'\1'),        # Remove space before empty lemma app notes.
        (r'}(\\edtext{[^}]})', r'} \1'),  # Add missing space between adjacent app. notes.
        (r' +', ' '),                     # Remove excessive whitespace.
        (r'} ([.,?!:;])', r'}\1'),        # Remove redundant space between closing brackets. and punctuation.
        (r'^ +', r''),                    # Remove leading space at beginning of line.
        (r' %$', '%'),                    # Remove trailing whitespace at paragraph end.
        ('\( ', r'('),                    # Remove trailing whitespace inside parenthesis.
        # NASTY!!!
        # quia\edtext{}{\lemma{\textnormal{quia}} => \edtext{quia}{\lemma{\textnormal{quia}}
        (r'(\w+)\\edtext{}{\\lemma{\1}', r'\\edtext{\1}{\\lemma{\1}'),


        # Replace anything wrapped in "..." with \enquote{...}. This is a bit dangerous as it
        # assumes that the editor always balances his quotes, and we cannot be sure of that.
        (r'"([^"]+?)"', r'\\enquote{\1}'),
    ]
    fname = tex_file.name
    orig_fname = fname + '.orig'

    # Rename original tex file into a .tex.orig file
    try:
        os.rename(fname, orig_fname)
    except IOError:
        raise IOError("Could not create temp file for cleaning.")

    # Open original file in read only mode
    try:
        fi = open(orig_fname, 'r')
    except IOError:
        raise IOError("Could not open file.")

    # Create a new file that will contain the clean text
    try:
        fo = open(fname, 'w')
    except IOError:
        raise IOError("Could not create temp file for cleaning.")

    with fi, fo:
        for line in fi.readlines():
            for pattern, replacement in patterns:
                line = re.sub(pattern, replacement, line)
            fo.write(line)

    # First check that the new file exists before deleting the old one
    if os.path.isfile(fname):
        try:
            os.remove(orig_fname)
        except IOError:
            logging.warning("Could not delete temp file. Continuing...")

    logging.info('Whitespace removed.')
    return fo


def compile_tex(tex_file, output_dir=False):
    """Convert a tex file to pdf with XeLaTeX.

    This requires `latexmk` and `xelatex`.

    Keyword Arguments:
    tex_file -- the tex file object to be converted.
    output_dir -- the directory where latexmk should put the created files. [default: tex file dir]

    Return: Output file object.
    """

    def read_output(pipe, func):
        for line in iter(pipe.readline, b''):
            func(line)
        pipe.close()

    def write_output(get):
        for line in iter(get, None):
            logging.info(line.decode('utf-8').replace('\n', ''))

    if not output_dir:
        output_dir = os.path.dirname(tex_file.name)

    logging.info(f"Start compilation of {tex_file.name}")

    process = subprocess.Popen(f'latexmk --output-directory={output_dir} --xelatex '
                               f'--halt-on-error {tex_file.name}',
                               stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                               shell=True, bufsize=1)
    q = queue.Queue()
    out_thread = threading.Thread(target=read_output, args=(process.stdout, q.put))
    err_thread = threading.Thread(target=read_output, args=(process.stderr, q.put))
    write_thread = threading.Thread(target=write_output, args=(q.get,))

    for t in (out_thread, err_thread, write_thread):
        t.start()

    process.wait()
    q.put(None)

    if process.returncode == 0:
        output_basename, _ = os.path.splitext(tex_file.name)
        return open(output_basename + '.pdf')
    else:
        logging.error('The compilation failed. See tex output above for mor info.')
        raise Exception('Latex compilation failed.')

def annotate_samewords(tex_file):
    process = subprocess.Popen(['samewords', tex_file.name],
                               stderr=subprocess.PIPE, stdout=subprocess.PIPE)
    out, err = process.communicate()
    with open(tex_file.name, mode='wb') as f:
        if not err:
            f.write(out)
        else:
            logging.error(f'The sameword annotation process returned the following error: '
                          f'{err.decode("utf-8")}')
    return tex_file

def select_xlst_script(trans_obj):
    """Determine which xslt should be used.

    If a URL is provided, the script is downloaded and stored temporarily. If a local file is
    provided, its location is used.

    Keyword Arguments:
    -- trans_obj: Required. The object of the text under processing.
    -- url: The url of an external script.
    -- local: The location of a local script.

    Return: Directory as string.
    """
    schema_info = trans_obj.lbp_schema_info
    if schema_info['type'] == 'critical':
        xslt_document_type = 'critical'
    else:
        xslt_document_type = 'diplomatic'
    xslt_version = schema_info['version']
    top = os.path.join(MODULE_DIR, 'xslt')
    if xslt_version in os.listdir(top):
        if xslt_document_type + '.xslt' in os.listdir(os.path.join(top, xslt_version)):
            return os.path.relpath(os.path.join(top, xslt_version, xslt_document_type)) + '.xslt'
        else:
            raise FileNotFoundError(f"The file '{xslt_document_type}.xslt' was not found in '\
                                    {os.path.join(top, xslt_version)}.")
    else:
        raise NotADirectoryError(f"A directory for version {xslt_version} was not found in {top}")





import lbppy
class Expression(lbppy.Resource):
    def structure_type(self): #returns resource identifier
        return self.values("http://scta.info/property/structureType")

    def manifestations(self):
        return self.values("http://scta.info/property/hasManifestation")

    def canonical_manifestation(self):
        return self.values("http://scta.info/property/hasCanonicalManifestation")


	#def canonical_manifestation? # returns boolean
	#	!canonical_manifestation.to_s.nil?
	#
	# translations are a subclass of manifestations for any kind of manifestation not in the original language
	# note that this currently means the manifestations methods, will not grab translation-manifestations,
	# these must be called with translations method
    def translations(self):
        return self.values("http://scta.info/property/hasTranslation")

    def canonical_translation(self):
        return self.values("http://scta.info/property/hasCanonicalTranslation")

    #def canonical_translation?
	#		!canonical_translation.to_s.nil?

	# cannonical transcriptions refers to the canonical trancription of the canonical manifestation
	#def canonical_transcription # returns single transcription as ResourceIdentifier
	#	manifestation = self.canonical_manifestation()
	#	if (manifestation != nil):
	#		return manifestation.resource.canonical_transcription
	#		end
	#	end
	#	def canonical_transcription? #returns boolean
	#		!canonical_transcription.nil?
	#	end

    def next(self): # returns resource identifier of next expression or nil
        return self.values("http://scta.info/property/next")

    def previous(self): #returns ResourceIdentifier or nil
        return self.values("http://scta.info/property/previous")

    #def order_number(self): # returns integer
		## TODO: consider changing property so that there is more symmetry here
	#	if structure_type.short_id == "structureBlock"
	#		value("http://scta.info/property/paragraphNumber").to_s.to_i
	#	else
	#			value("http://scta.info/property/totalOrderNumber").to_s.to_i
	#		end
	#	end

    #def status(self): #returns string
	#	return self.values("http://scta.info/property/status").to_s

    def top_level_expression(self): # returns resource identifier
        #TODO make sure this can handle different structure types
        return self.values("http://scta.info/property/isPartOfTopLevelExpression")

    def item_level_expression(self): # returns resource identifier
        #TODO make sure this can handle different structure types
        return self.values("http://scta.info/property/isPartOfStructureItem")

	#def level(self): # returns resource integer
		#same comment as earlier; this query does not actually return a uri,
		#but an litteral. We need to make sure the resource identifer can handle that
	#	values("http://scta.info/property/level").to_s.to_i

    def abbreviates(self): # returns array of ResourceIdentifiers
        return self.values("http://scta.info/property/abbreviates")

    def abbreviatedBy(self):
        return self.values("http://scta.info/property/abbreviatedBy")

    def references(self):
        return self.values("http://scta.info/property/references")

    def referencedBy(self):
        return self.values("http://scta.info/property/referencedBy")

    def copies(self):
        return self.values("http://scta.info/property/copies")

    def copiedBy(self):
        return self.values("http://scta.info/property/copiedBy")

    def mentions(self):
        return self.values("http://scta.info/property/mentions")

    def quotes(self):
        return self.values("http://scta.info/property/quotes")

    def quotedBy(self):
        return self.values("http://scta.info/property/quotedBy")

    def isRelatedTo(self):
        return self.values("http://scta.info/property/isRelatedTo")



import lbppy

resource = lbppy.Resource.find("http://scta.info/resource/lectio1/critical/transcription")

# call for file part with specified kwargs
#text = resource.file(**{"path": "doc", "partid": "l1-cpspfs"})
html = resource.file().transform("https://raw.githubusercontent.com/lombardpress/lombardpress-web/master/xslt/1.0.0/critical/plaintext.xsl")
print(html)



import lbppy
import urllib
from lxml import etree

class File():
    def __init__(self, file_path, transcription_type, confighash):
        self.file_path = file_path
        self.confighash = confighash
        if (self.confighash != None):
            self.stylesheets = self.confighash["stylesheets"]

        self.transcription = transcription_type
        #if (self.confighash != nil):
        #	self.xslt_dir = "#{@confighash[:xslt_base]}#{@xslt_version}/#{@transcription_type}/"

    def file(self):
        response = urllib.request.urlopen(self.file_path)
        # this conditional is needed for private repos requring basic auth
        #if (response.base_uri.to_s != self.file_path):
            #response = urllib.request.urlopen(self.file_path, {:http_basic_authentication => [self.confighash["git_username"], self.confighash["git_password"] ]})
        return response
    def lxmldoc(self):
        doc = etree.parse(self.file())
        return doc
    def validating_schema_version(self):
        result = self.lxmldoc().xpath("/tei:TEI/tei:teiHeader[1]/tei:encodingDesc[1]/tei:schemaRef[1]/@n", namespaces={"tei": "http://www.tei-c.org/ns/1.0"})
        if (len(result) > 0):
            return result[0].split("-").pop()
        else:
            return "default"

    def title(self):
        xmldoc = self.lxmldoc()
        title = xmldoc.xpath("/tei:TEI/tei:teiHeader/tei:fileDesc/tei:titleStmt/tei:title", namespaces={"tei": "http://www.tei-c.org/ns/1.0"})
        return title[0].text

    def author(self):
        xmldoc = self.lxmldoc()
        author = xmldoc.xpath("/tei:TEI/tei:teiHeader[1]/tei:fileDesc/tei:titleStmt[1]/tei:author", namespaces={"tei": "http://www.tei-c.org/ns/1.0"})
        return author[0].text

    def editor(self):
        xmldoc = self.lxmldoc()
        editor = xmldoc.xpath("/tei:TEI/tei:teiHeader[1]/tei:fileDesc/tei:titleStmt[1]/tei:editor", namespaces={"tei": "http://www.tei-c.org/ns/1.0"})
        return editor[0].text

# begin transform methods
    def transform(self, xslturl):
        raw = urllib.request.urlopen(xslturl)
        xsltdoc = etree.parse(raw)
        xmldoc = self.lxmldoc()
        result = xmldoc.xslt(xsltdoc)
        return result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""LombardPress print.

Usage:
  lbp_print (tex|pdf) [options] --local <file>...
  lbp_print (tex|pdf) [options] --scta <expression-id>...
  lbp_print recipe <recipe> [options]

Pull LBP-compliant files from SCTA repositories or use local, convert them into
tex or pdf.

Arguments:
  <file>                   Location of one or more local files to be processed.
  <expression-id>          The expression id of the items to be processed.

Multiple arguments are separated with whitespace.

Commands:
  tex                      Convert the xml to a tex-file.
  pdf                      Convert the xml to a tex-file and compile it into a
                           pdf.
  recipe <recipe>          Follow recipe in config file at <file> location.

Options:
  --scta                   Flag. When present, the <identifier> should be an
                           expression id of the SCTA database.
  --local                  Flag. When present, process local file indicated
                           by <file> argument.
  --xslt <file>            Use a custom xslt file in place of the default
                           supplied templates.
  --output, -o <dir>       Put results in the specified directory.
  --xslt-parameters <str>  Command line parameters that will be
                           passed to the XSLT script. Unfortunately, this only
                           works with one parameter at the moment.
                           Example: --xslt-parameters "key=value"
  --config-file <file>     Location of a config file in json format.
                           [default: ~/.lbp_print.json]
  -V, --verbosity <level>  Set verbosity. Possibilities: silent, info, debug
                           [default: info].
  -v, --version            Show version and exit.
  -h, --help               Show this help message and exit.
"""

import logging
import os
import json

from docopt import docopt

from lbp_print.core import LocalTranscription, RemoteTranscription, select_xlst_script, clean_tex, \
    convert_xml_to_tex, compile_tex, annotate_samewords
from lbp_print.__about__ import __version__

def load_config(filename):
    """Load and read in configuration from local config file.

    :return Dictionary of the configuration."""
    def expand_in_dict(key, dict):
        """Expand os user name in dict key."""
        if key in dict:
            if isinstance(dict[key], list):
                return [os.path.expanduser(item) for item in dict[key]]
            elif isinstance(dict[key], str):
                return os.path.expanduser(dict[key])

    try:
        with open(filename, mode='r') as f:
            try:
                conf = json.loads(f.read())
            except json.decoder.JSONDecodeError as e:
                logging.error(f"The config file {f.name} is incorrectly formatted.\n"
                              f"JSON deconding gave the following error: {e}")
                raise

            # Expand user commands in file arguments.
            for key in ['<file>', '--output', '--xslt']:
                if key in conf:
                    conf[key] = expand_in_dict(key, conf)

            return conf
    except FileNotFoundError:
        logging.warning(f'The config file {filename} was not found. Default settings will be used.')
        return {}

def merge(dict_1, dict_2):
    """Merge two dictionaries.
    Values that evaluate to true take priority over falsy values.
    `dict_1` takes priority over `dict_2`.
    """
    return dict((str(key), dict_1.get(key) or dict_2.get(key))
                for key in set(dict_2) | set(dict_1))

def main():

    logging.basicConfig(level='INFO', format="%(levelname)s: %(message)s")

    # Read command line arguments
    cl_args = docopt(__doc__, version=__version__)

    # Expand user dir for config file.
    cl_args['--config-file'] = os.path.expanduser(cl_args['--config-file'])

    # Read config file or recipe
    if cl_args['<recipe>']:
        ini_args = load_config(cl_args['<recipe>'])
    else:
        ini_args = load_config(cl_args['--config-file'])

    # Merge configurations, giving command line arguments priority over config file arguments
    args = merge(cl_args, ini_args)

    # Setup logging according to configuration
    logging.getLogger().setLevel(args['--verbosity'].upper())

    # Debug startup info
    logging.debug('Logging initialized.')
    logging.debug(args)
    logging.debug('App initialized.')

    # Determine output directory
    if args["--output"]:
        output_dir = args["--output"]
    else:
        output_dir = False

    # Initialize the object
    transcriptions = []
    if args["--scta"]:
        for num, exp in enumerate(args["<expression-id>"], 1):
            logging.info(f'Initializing {exp}. [{num}/{len(args["<expression-id>"])}]')
            transcriptions.append(RemoteTranscription(exp))
    elif args["--local"]:
        for num, exp in enumerate(args["<file>"], 1):
            logging.info(f'Initializing {exp}. [{num}/{len(args["<file>"])}]')
            transcriptions.append(LocalTranscription(exp))
    else:
        raise ValueError("Either provide an expression-id or a reference to a local file.")

    for num, item in enumerate(transcriptions, 1):
        # Determine xslt script file (either provided or selected based on the xml transcription)
        logging.info('-------')
        logging.info(f'Processing {item.input}. [{num}/{len(transcriptions)}]')
        if args["--xslt"]:
            xslt_candidate = args["--xslt"]
            if os.path.isfile(xslt_candidate):
                xslt_script = xslt_candidate
            else:
                raise FileNotFoundError(f"The xslt file supplied, {xslt_candidate}, was not found.")
        else:
            xslt_script = select_xlst_script(item)

        tex_file = convert_xml_to_tex(item.file.name, xslt_script, output_dir, args["--xslt-parameters"])

        # clean tex file
        # there could be an option for whether or not a person wants this white space cleaning to take effect
        output_file = clean_tex(tex_file)

        output_file = annotate_samewords(output_file)

        if args["pdf"]:
            output_file = compile_tex(tex_file, output_dir)

        logging.info('Results returned sucessfully.\n '
                     'The output file is located at %s' % output_file.name)



import sys
import lbppy


class Resource():
    def __init__(self, url, results):
        self.results = results


    @classmethod
    def find(cls, url):
        query_obj = lbppy.Query()
        cls.results = query_obj.query(url)
        return cls.create(cls, url, cls.results)
    def create(cls, url, results):
        type = ""
        for result in cls.results["results"]["bindings"]:
            if (result["p"]["value"] == "http://www.w3.org/1999/02/22-rdf-syntax-ns#type"):
                type = result["o"]["value"].split("/").pop()
        type = type.capitalize()

        klass = getattr(sys.modules['lbppy'], type)

        return klass(url, results)


    def values(self, predicate):
        predicates = []
        for result in self.results["results"]["bindings"]:
            if (result["p"]["value"] == predicate):
                predicates.append(lbppy.ResourceIdentifier(result["o"]["value"]))

        if len(predicates) > 1:
            return predicates
        elif len(predicates) == 1:
            return predicates[0]
        else:
            return "no results found"

    def title(self):
        return self.values("http://purl.org/dc/elements/1.1/title").to_s()
    def description(self):
        return self.values("http://purl.org/dc/elements/1.1/description")
    def has_parts(self):
        return self.values("http://purl.org/dc/terms/hasPart")
    def is_part_of(self):
        return self.values("http://purl.org/dc/terms/isPartOf")

    def type(self):
        return self.values("http://www.w3.org/1999/02/22-rdf-syntax-ns#type")
    def next(self):
        return self.values("http://scta.info/property/next")
    def previous(self):
        return self.values("http://scta.info/property/previous")
    def inbox(self):
        return self.values("http://www.w3.org/ns/ldp#inbox")



from SPARQLWrapper import SPARQLWrapper, JSON

class Query():
    def __init__(self):
        self.description = "I am a query object"

    def query(self, url):
        new_url = "<" + url + ">"
        sparql = SPARQLWrapper("http://sparql-staging.scta.info/ds/query")
        sparql.setQuery("""
            PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
            PREFIX sctap: <http://scta.info/property/>
            PREFIX sctar: <http://scta.info/resource/>
            SELECT ?p ?o
            WHERE { %s ?p ?o }
        """ % (new_url))
        sparql.setReturnFormat(JSON)
        results = sparql.query().convert()
        return results



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages


about = {}
with open(os.path.join('lbp_print', '__about__.py')) as fp:
    exec(fp.read(), about)

setup(name='lbp_print',
      version=about['__version__'],
      packages=find_packages(),
      dependency_links=[
          "git+https://github.com/lombardpress/lbppy.git@f272e44429fcbd3f19384471c9cfb3903100fb30#egg=lbppy",
      ],
      install_requires=[
          'docopt==0.6.2',
          'isodate==0.5.4',
          'lbppy>=0.0.0',
          'lxml==3.7.3',
          'pyparsing==2.2.0',
          'rdflib==4.2.2',
          'SPARQLWrapper==1.8.0',
          'untangle==1.1.0',
      ],
      classifiers=[
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.6',
          'Topic :: Text Processing :: Markup :: XML',
          'Topic :: Text Processing :: Markup :: LaTeX',
      ],
      entry_points={
          'console_scripts': ['lbp_print=lbp_print.cli:main']
      },

      description='Script for compiling LombardPress encoded XML documents to TeX and PDF.',
      url='https://github.com/stenskjaer/lbp_print',
      author='Michael Stenskjær Christensen',
      author_email='michael.stenskjaer@gmail.com',
      license='MIT',
)
